@@ -39,9 +39,6 @@
  * #SoupAddress represents the address of a TCP connection endpoint:
  * both the IP address and the port. (It is somewhat like an
  * object-oriented version of struct sockaddr.)
- *
- * If libsoup was built with IPv6 support, #SoupAddress will allow
- * both IPv4 and IPv6 addresses.
  **/
 
 enum {
@@ -61,20 +58,20 @@ typedef struct {
 
 	char *name, *physical;
 	guint port;
+
+	GMutex *lock;
+	GSList *async_lookups;
 } SoupAddressPrivate;
 #define SOUP_ADDRESS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOUP_TYPE_ADDRESS, SoupAddressPrivate))
 
 /* sockaddr generic macros */
 #define SOUP_SIN(priv) ((struct sockaddr_in *)priv->sockaddr)
-#ifdef HAVE_IPV6
 #define SOUP_SIN6(priv) ((struct sockaddr_in6 *)priv->sockaddr)
-#endif
 
 /* sockaddr family macros */
 #define SOUP_ADDRESS_GET_FAMILY(priv) (priv->sockaddr->sa_family)
 #define SOUP_ADDRESS_SET_FAMILY(priv, family) \
 	(priv->sockaddr->sa_family = family)
-#ifdef HAVE_IPV6
 #define SOUP_ADDRESS_FAMILY_IS_VALID(family) \
 	(family == AF_INET || family == AF_INET6)
 #define SOUP_ADDRESS_FAMILY_SOCKADDR_SIZE(family) \
@@ -83,15 +80,9 @@ typedef struct {
 #define SOUP_ADDRESS_FAMILY_DATA_SIZE(family) \
 	(family == AF_INET ? sizeof (struct in_addr) : \
 			     sizeof (struct in6_addr))
-#else
-#define SOUP_ADDRESS_FAMILY_IS_VALID(family) (family == AF_INET)
-#define SOUP_ADDRESS_FAMILY_SOCKADDR_SIZE(family) sizeof (struct sockaddr_in)
-#define SOUP_ADDRESS_FAMILY_DATA_SIZE(family) sizeof (struct in_addr)
-#endif
 
 /* sockaddr port macros */
 #define SOUP_ADDRESS_PORT_IS_VALID(port) (port >= 0 && port <= 65535)
-#ifdef HAVE_IPV6
 #define SOUP_ADDRESS_GET_PORT(priv) \
 	(priv->sockaddr->sa_family == AF_INET ? \
 		SOUP_SIN(priv)->sin_port : \
@@ -103,20 +94,12 @@ typedef struct {
 	else						\
 		SOUP_SIN6(priv)->sin6_port = port;	\
 	} G_STMT_END
-#else
-#define SOUP_ADDRESS_GET_PORT(priv) (SOUP_SIN(priv)->sin_port)
-#define SOUP_ADDRESS_SET_PORT(priv, port) (SOUP_SIN(priv)->sin_port = port)
-#endif
 
 /* sockaddr data macros */
-#ifdef HAVE_IPV6
 #define SOUP_ADDRESS_GET_DATA(priv) \
 	(priv->sockaddr->sa_family == AF_INET ? \
 		(gpointer)&SOUP_SIN(priv)->sin_addr : \
 		(gpointer)&SOUP_SIN6(priv)->sin6_addr)
-#else
-#define SOUP_ADDRESS_GET_DATA(priv) ((gpointer)&SOUP_SIN(priv)->sin_addr)
-#endif
 #define SOUP_ADDRESS_SET_DATA(priv, data, length) \
 	memcpy (SOUP_ADDRESS_GET_DATA (priv), data, length)
 
@@ -134,6 +117,9 @@ G_DEFINE_TYPE (SoupAddress, soup_address, G_TYPE_OBJECT)
 static void
 soup_address_init (SoupAddress *addr)
 {
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
+
+	priv->lock = g_mutex_new ();
 }
 
 static void
@@ -149,6 +135,8 @@ finalize (GObject *object)
 	if (priv->physical)
 		g_free (priv->physical);
 
+	g_mutex_free (priv->lock);
+
 	G_OBJECT_CLASS (soup_address_parent_class)->finalize (object);
 }
 
@@ -231,15 +219,6 @@ soup_address_class_init (SoupAddressClass *address_class)
 				      "sockaddr",
 				      "struct sockaddr for this address",
 				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
-#ifdef G_OS_WIN32
-	/* This hopefully is a good place to call WSAStartup */
-	{
-		WSADATA wsadata;
-		if (WSAStartup (MAKEWORD (2, 0), &wsadata) != 0)
-			g_error ("Windows Sockets could not be initialized");
-	}
-#endif
 }
 
 static GObject *
@@ -385,7 +364,7 @@ soup_address_new (const char *name, guint port)
  * Returns a #SoupAddress equivalent to @sa (or %NULL if @sa's
  * address family isn't supported)
  *
- * Return value: the new #SoupAddress
+ * Return value: (allow-none): the new #SoupAddress
  **/
 SoupAddress *
 soup_address_new_from_sockaddr (struct sockaddr *sa, int len)
@@ -405,10 +384,7 @@ soup_address_new_from_sockaddr (struct sockaddr *sa, int len)
  * @SOUP_ADDRESS_FAMILY_IPV4: an IPv4 address
  * @SOUP_ADDRESS_FAMILY_IPV6: an IPv6 address
  *
- * The supported address families. Note that the
- * %SOUP_ADDRESS_FAMILY_IPV6 constant is available even if libsoup was
- * built without IPv6 support, but attempting to create an IPv6
- * address will fail in that case.
+ * The supported address families.
  **/
 
 /**
@@ -427,7 +403,7 @@ soup_address_new_from_sockaddr (struct sockaddr *sa, int len)
  * for @family (or %NULL if @family isn't supported), suitable for
  * passing to soup_socket_server_new().
  *
- * Return value: the new #SoupAddress
+ * Return value: (allow-none): the new #SoupAddress
  **/
 SoupAddress *
 soup_address_new_any (SoupAddressFamily family, guint port)
@@ -447,7 +423,12 @@ soup_address_new_any (SoupAddressFamily family, guint port)
  *
  * Returns the hostname associated with @addr.
  *
- * Return value: the hostname, or %NULL if it is not known.
+ * This method is not thread-safe; if you call it while @addr is being
+ * resolved in another thread, it may return garbage. You can use
+ * soup_address_is_resolved() to safely test whether or not an address
+ * is resolved before fetching its name or address.
+ *
+ * Return value: (allow-none): the hostname, or %NULL if it is not known.
  **/
 const char *
 soup_address_get_name (SoupAddress *addr)
@@ -465,7 +446,12 @@ soup_address_get_name (SoupAddress *addr)
  * Returns the sockaddr associated with @addr, with its length in
  * *@len. If the sockaddr is not yet known, returns %NULL.
  *
- * Return value: the sockaddr, or %NULL
+ * This method is not thread-safe; if you call it while @addr is being
+ * resolved in another thread, it may return garbage. You can use
+ * soup_address_is_resolved() to safely test whether or not an address
+ * is resolved before fetching its name or address.
+ *
+ * Return value: (allow-none): the sockaddr, or %NULL
  **/
 struct sockaddr *
 soup_address_get_sockaddr (SoupAddress *addr, int *len)
@@ -490,6 +476,7 @@ soup_address_make_inet_address (SoupAddress *addr)
 	gsa = g_socket_address_new_from_native (priv->sockaddr,
 						SOUP_ADDRESS_FAMILY_SOCKADDR_SIZE (SOUP_ADDRESS_GET_FAMILY (priv)));
 	gia = g_inet_socket_address_get_address ((GInetSocketAddress *)gsa);
+	g_object_ref (gia);
 	g_object_unref (gsa);
 	return gia;
 }
@@ -501,7 +488,12 @@ soup_address_make_inet_address (SoupAddress *addr)
  * Returns the physical address associated with @addr as a string.
  * (Eg, "127.0.0.1"). If the address is not yet known, returns %NULL.
  *
- * Return value: the physical address, or %NULL
+ * This method is not thread-safe; if you call it while @addr is being
+ * resolved in another thread, it may return garbage. You can use
+ * soup_address_is_resolved() to safely test whether or not an address
+ * is resolved before fetching its name or address.
+ *
+ * Return value: (allow-none): the physical address, or %NULL
  **/
 const char *
 soup_address_get_physical (SoupAddress *addr)
@@ -597,71 +589,70 @@ update_name (SoupAddress *addr, const char *name, GError *error)
 }
 
 typedef struct {
-	SoupAddress         *addr;
 	SoupAddressCallback  callback;
 	gpointer             callback_data;
-	gboolean             lookup_name;
-
-	GMainContext        *async_context;
-	GCancellable        *cancellable;
-	guint                status;
 } SoupAddressResolveAsyncData;
 
 static void
-complete_resolve_async (SoupAddressResolveAsyncData *res_data)
+complete_resolve_async (SoupAddress *addr, guint status)
 {
-	SoupAddress *addr = res_data->addr;
-	SoupAddressCallback callback = res_data->callback;
-	gpointer callback_data = res_data->callback_data;
+	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
+	SoupAddressResolveAsyncData *res_data;
+	GSList *lookups, *l;
 
-	if (callback)
-		callback (addr, res_data->status, callback_data);
+	lookups = priv->async_lookups;
+	priv->async_lookups = NULL;
 
-	if (res_data->async_context)
-		g_main_context_unref (res_data->async_context);
-	if (res_data->cancellable)
-		g_object_unref (res_data->cancellable);
+	for (l = lookups; l; l = l->next) {
+		res_data = l->data;
+
+		if (res_data->callback) {
+			res_data->callback (addr, status,
+					    res_data->callback_data);
+		}
+		g_slice_free (SoupAddressResolveAsyncData, res_data);
+	}
+	g_slist_free (lookups);
 
 	g_object_unref (addr);
-	g_slice_free (SoupAddressResolveAsyncData, res_data);
 }
 
 static void
 lookup_resolved (GObject *source, GAsyncResult *result, gpointer user_data)
 {
 	GResolver *resolver = G_RESOLVER (source);
-	SoupAddressResolveAsyncData *res_data = user_data;
-	SoupAddress *addr = res_data->addr;
+	SoupAddress *addr = user_data;
 	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 	GError *error = NULL;
+	guint status;
 
 	if (!priv->sockaddr) {
 		GList *addrs;
 
 		addrs = g_resolver_lookup_by_name_finish (resolver, result,
 							  &error);
-		res_data->status = update_addrs (addr, addrs, error);
+		status = update_addrs (addr, addrs, error);
 		g_resolver_free_addresses (addrs);
 	} else if (!priv->name) {
 		char *name;
 
 		name = g_resolver_lookup_by_address_finish (resolver, result,
 							    &error);
-		res_data->status = update_name (addr, name, error);
+		status = update_name (addr, name, error);
 		g_free (name);
 	} else
-		res_data->status = SOUP_STATUS_OK;
+		status = SOUP_STATUS_OK;
 
 	if (error)
 		g_error_free (error);
 
-	complete_resolve_async (res_data);
+	complete_resolve_async (addr, status);
 }
 
 static gboolean
-idle_complete_resolve (gpointer res_data)
+idle_complete_resolve (gpointer addr)
 {
-	complete_resolve_async (res_data);
+	complete_resolve_async (addr, SOUP_STATUS_OK);
 	return FALSE;
 }
 
@@ -681,7 +672,7 @@ idle_complete_resolve (gpointer res_data)
  * @addr: a #SoupAddress
  * @async_context: the #GMainContext to call @callback from
  * @cancellable: a #GCancellable object, or %NULL
- * @callback: callback to call with the result
+ * @callback: (scope async): callback to call with the result
  * @user_data: data for @callback
  *
  * Asynchronously resolves the missing half of @addr (its IP address
@@ -692,6 +683,12 @@ idle_complete_resolve (gpointer res_data)
  * If @cancellable is non-%NULL, it can be used to cancel the
  * resolution. @callback will still be invoked in this case, with a
  * status of %SOUP_STATUS_CANCELLED.
+ *
+ * It is safe to call this more than once on a given address, from the
+ * same thread, with the same @async_context (and doing so will not
+ * result in redundant DNS queries being made). But it is not safe to
+ * call from multiple threads, or with different @async_contexts, or
+ * mixed with calls to soup_address_resolve_sync().
  **/
 void
 soup_address_resolve_async (SoupAddress *addr, GMainContext *async_context,
@@ -701,46 +698,56 @@ soup_address_resolve_async (SoupAddress *addr, GMainContext *async_context,
 	SoupAddressPrivate *priv;
 	SoupAddressResolveAsyncData *res_data;
 	GResolver *resolver;
+	gboolean already_started;
 
 	g_return_if_fail (SOUP_IS_ADDRESS (addr));
 	priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 	g_return_if_fail (priv->name || priv->sockaddr);
 
+	/* We don't need to do locking here because the async case is
+	 * not intended to be thread-safe.
+	 */
+
+	if (priv->name && priv->sockaddr && !callback)
+		return;
+
 	res_data = g_slice_new0 (SoupAddressResolveAsyncData);
-	res_data->addr          = g_object_ref (addr);
-	res_data->callback      = callback;
+	res_data->callback = callback;
 	res_data->callback_data = user_data;
 
+	already_started = priv->async_lookups != NULL;
+	priv->async_lookups = g_slist_prepend (priv->async_lookups, res_data);
+
+	if (already_started)
+		return;
+
+	g_object_ref (addr);
+
 	if (priv->name && priv->sockaddr) {
-		res_data->status = SOUP_STATUS_OK;
-		soup_add_completion (async_context, idle_complete_resolve, res_data);
+		soup_add_completion (async_context, idle_complete_resolve, addr);
 		return;
 	}
 
 	resolver = g_resolver_get_default ();
-
 	if (async_context && async_context != g_main_context_default ())
 		g_main_context_push_thread_default (async_context);
 
 	if (priv->name) {
-		res_data->lookup_name = TRUE;
 		g_resolver_lookup_by_name_async (resolver, priv->name,
 						 cancellable,
-						 lookup_resolved, res_data);
+						 lookup_resolved, addr);
 	} else {
 		GInetAddress *gia;
 
-		res_data->lookup_name = FALSE;
 		gia = soup_address_make_inet_address (addr);
 		g_resolver_lookup_by_address_async (resolver, gia,
 						    cancellable,
-						    lookup_resolved, res_data);
+						    lookup_resolved, addr);
 		g_object_unref (gia);
 	}
 
 	if (async_context && async_context != g_main_context_default ())
 		g_main_context_pop_thread_default (async_context);
-
 	g_object_unref (resolver);
 }
 
@@ -756,6 +763,11 @@ soup_address_resolve_async (SoupAddress *addr, GMainContext *async_context,
  * resolution. soup_address_resolve_sync() will then return a status
  * of %SOUP_STATUS_CANCELLED.
  *
+ * It is safe to call this more than once, even from different
+ * threads, but it is not safe to mix calls to
+ * soup_address_resolve_sync() with calls to
+ * soup_address_resolve_async() on the same address.
+ *
  * Return value: %SOUP_STATUS_OK, %SOUP_STATUS_CANT_RESOLVE, or
  * %SOUP_STATUS_CANCELLED.
  **/
@@ -773,25 +785,39 @@ soup_address_resolve_sync (SoupAddress *addr, GCancellable *cancellable)
 
 	resolver = g_resolver_get_default ();
 
+	/* We could optimize this to avoid multiple lookups the same
+	 * way _resolve_async does, but we don't currently. So, first
+	 * lock the mutex to ensure we have a consistent view of
+	 * priv->sockaddr and priv->name, unlock it around the
+	 * blocking op, and then re-lock it to modify @addr.
+	 */
+	g_mutex_lock (priv->lock);
 	if (!priv->sockaddr) {
 		GList *addrs;
 
+		g_mutex_unlock (priv->lock);
 		addrs = g_resolver_lookup_by_name (resolver, priv->name,
 						   cancellable, &error);
+		g_mutex_lock (priv->lock);
+
 		status = update_addrs (addr, addrs, error);
 		g_resolver_free_addresses (addrs);
 	} else if (!priv->name) {
 		GInetAddress *gia;
 		char *name;
 
+		g_mutex_unlock (priv->lock);
 		gia = soup_address_make_inet_address (addr);
 		name = g_resolver_lookup_by_address (resolver, gia,
 						     cancellable, &error);
 		g_object_unref (gia);
+		g_mutex_lock (priv->lock);
+
 		status = update_name (addr, name, error);
 		g_free (name);
 	} else
 		status = SOUP_STATUS_OK;
+	g_mutex_unlock (priv->lock);
 
 	if (error)
 		g_error_free (error);
@@ -804,7 +830,9 @@ soup_address_resolve_sync (SoupAddress *addr, GCancellable *cancellable)
  * soup_address_is_resolved:
  * @addr: a #SoupAddress
  *
- * Tests if @addr has already been resolved.
+ * Tests if @addr has already been resolved. Unlike the other
+ * #SoupAddress "get" methods, this is safe to call when @addr might
+ * be being resolved in another thread.
  *
  * Return value: %TRUE if @addr has been resolved.
  **/
@@ -812,11 +840,16 @@ gboolean
 soup_address_is_resolved (SoupAddress *addr)
 {
 	SoupAddressPrivate *priv;
+	gboolean resolved;
 
 	g_return_val_if_fail (SOUP_IS_ADDRESS (addr), FALSE);
 	priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 
-	return priv->sockaddr && priv->name;
+	g_mutex_lock (priv->lock);
+	resolved = priv->sockaddr && priv->name;
+	g_mutex_unlock (priv->lock);
+
+	return resolved;
 }
 
 /**
