@@ -100,6 +100,7 @@ soup_message_flags_get_type (void)
 			{ SOUP_MESSAGE_NO_REDIRECT, "SOUP_MESSAGE_NO_REDIRECT", "no-redirect" },
 			{ SOUP_MESSAGE_OVERWRITE_CHUNKS, "SOUP_MESSAGE_OVERWRITE_CHUNKS", "overwrite-chunks" },
 			{ SOUP_MESSAGE_CONTENT_DECODED, "SOUP_MESSAGE_CONTENT_DECODED", "content-decoded" },
+			{ SOUP_MESSAGE_CERTIFICATE_TRUSTED, "SOUP_MESSAGE_CERTIFICATE_TRUSTED", "certificate-trusted" },
 			{ 0, NULL, NULL }
 		};
 		etype = g_flags_register_static (g_intern_static_string ("SoupMessageFlags"), values);
@@ -186,6 +187,7 @@ soup_ssl_error_get_type (void)
 			{ SOUP_SSL_ERROR_HANDSHAKE_NEEDS_READ, "SOUP_SSL_ERROR_HANDSHAKE_NEEDS_READ", "handshake-needs-read" },
 			{ SOUP_SSL_ERROR_HANDSHAKE_NEEDS_WRITE, "SOUP_SSL_ERROR_HANDSHAKE_NEEDS_WRITE", "handshake-needs-write" },
 			{ SOUP_SSL_ERROR_CERTIFICATE, "SOUP_SSL_ERROR_CERTIFICATE", "certificate" },
+			{ SOUP_SSL_ERROR_HANDSHAKE_FAILED, "SOUP_SSL_ERROR_HANDSHAKE_FAILED", "handshake-failed" },
 			{ 0, NULL, NULL }
 		};
 		etype = g_enum_register_static (g_intern_static_string ("SoupSSLError"), values);
