@@ -12,6 +12,10 @@ G_BEGIN_DECLS
 GType soup_address_family_get_type (void) G_GNUC_CONST;
 #define SOUP_TYPE_ADDRESS_FAMILY (soup_address_family_get_type ())
 
+/* enumerations from "soup-cookie-jar.h" */
+GType soup_cookie_jar_accept_policy_get_type (void) G_GNUC_CONST;
+#define SOUP_TYPE_COOKIE_JAR_ACCEPT_POLICY (soup_cookie_jar_accept_policy_get_type ())
+
 /* enumerations from "soup-date.h" */
 GType soup_date_format_get_type (void) G_GNUC_CONST;
 #define SOUP_TYPE_DATE_FORMAT (soup_date_format_get_type ())
