@@ -158,9 +158,12 @@ soup_headers_clean_for_10 (SoupMessageHeaders *hdrs)
  * @str: the header string (including the trailing blank line)
  * @len: length of @str up to (but not including) the terminating blank line.
  * @req_headers: #SoupMessageHeaders to store the header values in
- * @req_method: if non-%NULL, will be filled in with the request method
- * @req_path: if non-%NULL, will be filled in with the request path
- * @ver: if non-%NULL, will be filled in with the HTTP version
+ * @req_method: (out) (allow-none): if non-%NULL, will be filled in with the
+ * request method
+ * @req_path: (out) (allow-none): if non-%NULL, will be filled in with the
+ * request path
+ * @ver: (out) (allow-none): if non-%NULL, will be filled in with the HTTP
+ * version
  *
  * Parses the headers of an HTTP request in @str and stores the
  * results in @req_method, @req_path, @ver, and @req_headers.
@@ -188,10 +191,12 @@ soup_headers_parse_request (const char          *str,
 	/* RFC 2616 4.1 "servers SHOULD ignore any empty line(s)
 	 * received where a Request-Line is expected."
 	 */
-	while (*str == '\r' || *str == '\n') {
+	while ((*str == '\r' || *str == '\n') && len > 0) {
 		str++;
 		len--;
 	}
+	if (!len)
+		return SOUP_STATUS_BAD_REQUEST;
 
 	/* RFC 2616 19.3 "[servers] SHOULD accept any amount of SP or
 	 * HT characters between [Request-Line] fields"
@@ -262,10 +267,12 @@ soup_headers_parse_request (const char          *str,
 /**
  * soup_headers_parse_status_line:
  * @status_line: an HTTP Status-Line
- * @ver: if non-%NULL, will be filled in with the HTTP version
- * @status_code: if non-%NULL, will be filled in with the status code
- * @reason_phrase: if non-%NULL, will be filled in with the reason
- * phrase
+ * @ver: (out) (allow-none): if non-%NULL, will be filled in with the HTTP
+ * version
+ * @status_code: (out) (allow-none): if non-%NULL, will be filled in with
+ * the status code
+ * @reason_phrase: (out) (allow-none): if non-%NULL, will be filled in with
+ * the reason phrase
  *
  * Parses the HTTP Status-Line string in @status_line into @ver,
  * @status_code, and @reason_phrase. @status_line must be terminated by
@@ -337,10 +344,12 @@ soup_headers_parse_status_line (const char       *status_line,
  * @str: the header string (including the trailing blank line)
  * @len: length of @str up to (but not including) the terminating blank line.
  * @headers: #SoupMessageheaders to store the header values in
- * @ver: if non-%NULL, will be filled in with the HTTP version
- * @status_code: if non-%NULL, will be filled in with the status code
- * @reason_phrase: if non-%NULL, will be filled in with the reason
- * phrase
+ * @ver: (out) (allow-none): if non-%NULL, will be filled in with the HTTP
+ * version
+ * @status_code: (out) (allow-none): if non-%NULL, will be filled in with
+ * the status code
+ * @reason_phrase: (out) (allow-none): if non-%NULL, will be filled in with
+ * the reason phrase
  *
  * Parses the headers of an HTTP response in @str and stores the
  * results in @ver, @status_code, @reason_phrase, and @headers.
@@ -361,6 +370,17 @@ soup_headers_parse_response (const char          *str,
 
 	g_return_val_if_fail (str && *str, FALSE);
 
+	/* Workaround for broken servers that send extra line breaks
+	 * after a response, which we then see prepended to the next
+	 * response on that connection.
+	 */
+	while ((*str == '\r' || *str == '\n') && len > 0) {
+		str++;
+		len--;
+	}
+	if (!len)
+		return FALSE;
+
 	if (!soup_headers_parse (str, len, headers)) 
 		return FALSE;
 
@@ -491,8 +511,8 @@ sort_by_qval (const void *a, const void *b)
 /**
  * soup_header_parse_quality_list:
  * @header: a header value
- * @unacceptable: on return, will contain a list of unacceptable
- * values
+ * @unacceptable: (out) (allow-none): on return, will contain a list of
+ * unacceptable values
  *
  * Parses a header whose content is a list of items with optional
  * "qvalue"s (eg, Accept, Accept-Charset, Accept-Encoding,
@@ -728,7 +748,7 @@ parse_param_list (const char *header, char delim)
  * for giving UTF8-encoded filenames in the Content-Disposition
  * header).
  *
- * Return value: a #GHashTable of list elements, which can be freed
+ * Return value: (element-type utf8 utf8): a #GHashTable of list elements, which can be freed
  * with soup_header_free_param_list().
  **/
 GHashTable *
@@ -753,8 +773,8 @@ soup_header_parse_param_list (const char *header)
  * for giving UTF8-encoded filenames in the Content-Disposition
  * header).
  *
- * Return value: a #GHashTable of list elements, which can be freed
- * with soup_header_free_param_list().
+ * Return value: (element-type utf8 utf8): a #GHashTable of list
+ * elements, which can be freed with soup_header_free_param_list().
  *
  * Since: 2.24
  **/
@@ -768,7 +788,7 @@ soup_header_parse_semi_param_list (const char *header)
 
 /**
  * soup_header_free_param_list:
- * @param_list: a #GHashTable returned from soup_header_parse_param_list()
+ * @param_list: (element-type utf8 utf8): a #GHashTable returned from soup_header_parse_param_list()
  * or soup_header_parse_semi_param_list()
  *
  * Frees @param_list.
@@ -782,21 +802,42 @@ soup_header_free_param_list (GHashTable *param_list)
 }
 
 static void
-append_param_rfc2231 (GString *string, const char *value)
+append_param_rfc2231 (GString    *string,
+		      const char *name,
+		      const char *value)
 {
 	char *encoded;
 
+	g_string_append (string, name);
 	g_string_append (string, "*=UTF-8''");
 	encoded = soup_uri_encode (value, " *'%()<>@,;:\\\"/[]?=");
 	g_string_append (string, encoded);
 	g_free (encoded);
 }
 
-static void
-append_param_quoted (GString *string, const char *value)
+/**
+ * soup_header_g_string_append_param_quoted:
+ * @string: a #GString being used to construct an HTTP header value
+ * @name: a parameter name
+ * @value: a parameter value
+ *
+ * Appends something like <literal>@name="@value"</literal> to
+ * @string, taking care to escape any quotes or backslashes in @value.
+ *
+ * Since: 2.32
+ **/
+void
+soup_header_g_string_append_param_quoted (GString    *string,
+					  const char *name,
+					  const char *value)
 {
 	int len;
 
+	g_return_if_fail (string != NULL);
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (value != NULL);
+
+	g_string_append (string, name);
 	g_string_append (string, "=\"");
 	while (*value) {
 		while (*value == '\\' || *value == '"') {
@@ -816,9 +857,9 @@ append_param_quoted (GString *string, const char *value)
  * @name: a parameter name
  * @value: a parameter value, or %NULL
  *
- * Appends something like <literal>@name="@value"</literal> to
- * @string, taking care to appropriately escape any quotes or
- * backslashes in @value.
+ * Appends something like <literal>@name=@value</literal> to @string,
+ * taking care to quote @value if needed, and if so, to escape any
+ * quotes or backslashes in @value.
  *
  * Alternatively, if @value is a non-ASCII UTF-8 string, it will be
  * appended using RFC2231 syntax. Although in theory this is supposed
@@ -831,26 +872,38 @@ append_param_quoted (GString *string, const char *value)
  * Since: 2.26
  **/
 void
-soup_header_g_string_append_param (GString *string, const char *name,
+soup_header_g_string_append_param (GString    *string,
+				   const char *name,
 				   const char *value)
 {
 	const char *v;
+	gboolean token = TRUE;
 
 	g_return_if_fail (string != NULL);
 	g_return_if_fail (name != NULL);
 
-	g_string_append (string, name);
-	if (!value)
+	if (!value) {
+		g_string_append (string, name);
 		return;
+	}
 
 	for (v = value; *v; v++) {
 		if (*v & 0x80) {
 			if (g_utf8_validate (value, -1, NULL)) {
-				append_param_rfc2231 (string, value);
+				append_param_rfc2231 (string, name, value);
 				return;
-			} else
+			} else {
+				token = FALSE;
 				break;
-		}
+			}
+		} else if (!soup_char_is_token (*v))
+			token = FALSE;
 	}
-	append_param_quoted (string, value);
+
+	if (token) {
+		g_string_append (string, name);
+		g_string_append_c (string, '=');
+		g_string_append (string, value);
+	} else
+		soup_header_g_string_append_param_quoted (string, name, value);
 }
