@@ -56,6 +56,18 @@ parse_response_headers (SoupMessage *req,
 	if (*encoding == SOUP_ENCODING_UNRECOGNIZED)
 		return SOUP_STATUS_MALFORMED;
 
+        /* mirror Mozilla here:
+         * (see netwerk/protocol/http/src/nsHttpTransaction.cpp for details)
+         *
+         * HTTP servers have been known to send erroneous Content-Length headers.
+         * So, unless the connection is persistent, we must make allowances for a
+         * possibly invalid Content-Length header. Thus, if NOT persistent, we 
+         * simply accept the whole message.
+         */
+        if (*encoding == SOUP_ENCODING_CONTENT_LENGTH &&
+            !soup_message_is_keepalive (req))
+                *encoding = SOUP_ENCODING_EOF;
+
 	return SOUP_STATUS_OK;
 }
 
@@ -84,6 +96,13 @@ get_request_headers (SoupMessage *req, GString *header,
 		 * just the path.
 		 */
 		uri_string = soup_uri_to_string (uri, !proxy);
+
+		if (proxy && uri->fragment) {
+			/* Strip fragment */
+			char *fragment = strchr (uri_string, '#');
+			if (fragment)
+				*fragment = '\0';
+		}
 	}
 
 	if (priv->http_version == SOUP_HTTP_1_0) {
