@@ -92,7 +92,7 @@
  **/
 
 static void append_uri_encoded (GString *str, const char *in, const char *extra_enc_chars);
-static char *uri_decoded_copy (const char *str, int length);
+static char *uri_decoded_copy (const char *str, int length, gboolean fixup);
 static char *uri_normalized_copy (const char *str, int length, const char *unescape_extra, gboolean fixup);
 
 gpointer _SOUP_URI_SCHEME_HTTP, _SOUP_URI_SCHEME_HTTPS;
@@ -100,7 +100,7 @@ gpointer _SOUP_URI_SCHEME_FTP;
 gpointer _SOUP_URI_SCHEME_FILE, _SOUP_URI_SCHEME_DATA;
 
 static inline const char *
-soup_uri_get_scheme (const char *scheme, int len)
+soup_uri_parse_scheme (const char *scheme, int len)
 {
 	if (len == 4 && !g_ascii_strncasecmp (scheme, "http", len)) {
 		return SOUP_URI_SCHEME_HTTP;
@@ -146,24 +146,43 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 	const char *end, *hash, *colon, *at, *path, *question;
 	const char *p, *hostend;
 	gboolean remove_dot_segments = TRUE;
+	int len;
+
+	/* First some cleanup steps (which are supposed to all be no-ops,
+	 * but...). Skip initial whitespace, strip out internal tabs and
+	 * line breaks, and ignore trailing whitespace.
+	 */
+	while (g_ascii_isspace (*uri_string))
+		uri_string++;
+
+	len = strcspn (uri_string, "\t\n\r");
+	if (uri_string[len]) {
+		char *clean = g_malloc (strlen (uri_string) + 1), *d;
+		const char *s;
+
+		for (s = uri_string, d = clean; *s; s++) {
+			if (*s != '\t' && *s != '\n' && *s != '\r')
+				*d++ = *s;
+		}
+		*d = '\0';
+
+		uri = soup_uri_new_with_base (base, clean);
+		g_free (clean);
+		return uri;
+	}
+	end = uri_string + len;
+	while (end > uri_string && g_ascii_isspace (end[-1]))
+		end--;
 
 	uri = g_slice_new0 (SoupURI);
 
-	/* See RFC 3986 for details. IF YOU CHANGE ANYTHING IN THIS
-	 * FUNCTION, RUN tests/uri-parsing AFTERWARDS.
-	 */
-
 	/* Find fragment. */
-	end = hash = strchr (uri_string, '#');
-	if (hash && hash[1]) {
-		uri->fragment = uri_normalized_copy (hash + 1, strlen (hash + 1),
-						     NULL, FALSE);
-		if (!uri->fragment) {
-			soup_uri_free (uri);
-			return NULL;
-		}
-	} else
-		end = uri_string + strlen (uri_string);
+	hash = strchr (uri_string, '#');
+	if (hash) {
+		uri->fragment = uri_normalized_copy (hash + 1, end - hash + 1,
+						     NULL, TRUE);
+		end = hash;
+	}
 
 	/* Find scheme: initial [a-z+.-]* substring until ":" */
 	p = uri_string;
@@ -172,15 +191,11 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 		p++;
 
 	if (p > uri_string && *p == ':') {
-		uri->scheme = soup_uri_get_scheme (uri_string, p - uri_string);
-		if (!uri->scheme) {
-			soup_uri_free (uri);
-			return NULL;
-		}
+		uri->scheme = soup_uri_parse_scheme (uri_string, p - uri_string);
 		uri_string = p + 1;
 	}
 
-	if (!*uri_string && !base)
+	if (uri_string == end && !base && !uri->fragment)
 		return uri;
 
 	/* Check for authority */
@@ -188,27 +203,23 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 		uri_string += 2;
 
 		path = uri_string + strcspn (uri_string, "/?#");
+		if (path > end)
+			path = end;
 		at = strchr (uri_string, '@');
 		if (at && at < path) {
 			colon = strchr (uri_string, ':');
 			if (colon && colon < at) {
 				uri->password = uri_decoded_copy (colon + 1,
-								  at - colon - 1);
-				if (!uri->password) {
-					soup_uri_free (uri);
-					return NULL;
-				}
+								  at - colon - 1,
+								  TRUE);
 			} else {
 				uri->password = NULL;
 				colon = at;
 			}
 
 			uri->user = uri_decoded_copy (uri_string,
-						      colon - uri_string);
-			if (!uri->user) {
-				soup_uri_free (uri);
-				return NULL;
-			}
+						      colon - uri_string,
+						      TRUE);
 			uri_string = at + 1;
 		} else
 			uri->user = uri->password = NULL;
@@ -230,11 +241,8 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 			hostend = colon ? colon : path;
 		}
 
-		uri->host = uri_decoded_copy (uri_string, hostend - uri_string);
-		if (!uri->host) {
-			soup_uri_free (uri);
-			return NULL;
-		}
+		uri->host = uri_decoded_copy (uri_string, hostend - uri_string,
+					      TRUE);
 
 		if (colon && colon != path - 1) {
 			char *portend;
@@ -254,23 +262,15 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 		uri->query = uri_normalized_copy (question + 1,
 						  end - (question + 1),
 						  NULL, TRUE);
-		if (!uri->query) {
-			soup_uri_free (uri);
-			return NULL;
-		}
 		end = question;
 	}
 
 	if (end != uri_string) {
 		uri->path = uri_normalized_copy (uri_string, end - uri_string,
 						 NULL, TRUE);
-		if (!uri->path) {
-			soup_uri_free (uri);
-			return NULL;
-		}
 	}
 
-	/* Apply base URI. Again, this is spelled out in RFC 3986. */
+	/* Apply base URI. This is spelled out in RFC 3986. */
 	if (base && !uri->scheme && uri->host)
 		uri->scheme = base->scheme;
 	else if (base && !uri->scheme) {
@@ -559,37 +559,14 @@ soup_uri_free (SoupURI *uri)
 	g_slice_free (SoupURI, uri);
 }
 
-/* From RFC 3986 */
-#define SOUP_URI_UNRESERVED  0
-#define SOUP_URI_PCT_ENCODED 1
-#define SOUP_URI_GEN_DELIMS  2
-#define SOUP_URI_SUB_DELIMS  4
-static const char uri_encoded_char[] = {
-	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  /* 0x00 - 0x0f */
-	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  /* 0x10 - 0x1f */
-	1, 4, 1, 2, 4, 1, 4, 4, 4, 4, 4, 4, 4, 0, 0, 2,  /*  !"#$%&'()*+,-./ */
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 4, 1, 4, 1, 2,  /* 0123456789:;<=>? */
-	2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* @ABCDEFGHIJKLMNO */
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 1, 2, 1, 0,  /* PQRSTUVWXYZ[\]^_ */
-	1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* `abcdefghijklmno */
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1,  /* pqrstuvwxyz{|}~  */
-	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
-};
-
 static void
 append_uri_encoded (GString *str, const char *in, const char *extra_enc_chars)
 {
 	const unsigned char *s = (const unsigned char *)in;
 
 	while (*s) {
-		if ((uri_encoded_char[*s] & (SOUP_URI_PCT_ENCODED | SOUP_URI_GEN_DELIMS)) ||
+		if (soup_char_is_uri_percent_encoded (*s) ||
+		    soup_char_is_uri_gen_delims (*s) ||
 		    (extra_enc_chars && strchr (extra_enc_chars, *s)))
 			g_string_append_printf (str, "%%%02X", (int)*s++);
 		else
@@ -626,7 +603,7 @@ soup_uri_encode (const char *part, const char *escape_extra)
 #define HEXCHAR(s) ((XDIGIT (s[1]) << 4) + XDIGIT (s[2]))
 
 static char *
-uri_decoded_copy (const char *part, int length)
+uri_decoded_copy (const char *part, int length, gboolean fixup)
 {
 	unsigned char *s, *d;
 	char *decoded = g_strndup (part, length);
@@ -636,8 +613,12 @@ uri_decoded_copy (const char *part, int length)
 		if (*s == '%') {
 			if (!g_ascii_isxdigit (s[1]) ||
 			    !g_ascii_isxdigit (s[2])) {
-				g_free (decoded);
-				return NULL;
+				if (!fixup) {
+					g_free (decoded);
+					return NULL;
+				}
+				*d++ = *s;
+				continue;
 			}
 			*d++ = HEXCHAR (s);
 			s += 2;
@@ -660,7 +641,7 @@ uri_decoded_copy (const char *part, int length)
 char *
 soup_uri_decode (const char *part)
 {
-	return uri_decoded_copy (part, strlen (part));
+	return uri_decoded_copy (part, strlen (part), FALSE);
 }
 
 static char *
@@ -676,12 +657,16 @@ uri_normalized_copy (const char *part, int length,
 		if (*s == '%') {
 			if (!g_ascii_isxdigit (s[1]) ||
 			    !g_ascii_isxdigit (s[2])) {
-				g_free (normalized);
-				return NULL;
+				if (!fixup) {
+					g_free (normalized);
+					return NULL;
+				}
+				*d++ = *s;
+				continue;
 			}
 
 			c = HEXCHAR (s);
-			if (uri_encoded_char[c] == SOUP_URI_UNRESERVED ||
+			if (soup_char_is_uri_unreserved (c) ||
 			    (unescape_extra && strchr (unescape_extra, c))) {
 				*d++ = c;
 				s += 2;
@@ -776,6 +761,22 @@ soup_uri_uses_default_port (SoupURI *uri)
  * against the value of a #SoupURI's <structfield>scheme</structfield>
  **/
 
+/**
+ * soup_uri_get_scheme:
+ * @uri: a #SoupURI
+ *
+ * Gets @uri's scheme.
+ *
+ * Return value: @uri's scheme.
+ *
+ * Since: 2.32
+ **/
+const char *
+soup_uri_get_scheme (SoupURI *uri)
+{
+	return uri->scheme;
+}
+
 /**
  * soup_uri_set_scheme:
  * @uri: a #SoupURI
@@ -787,10 +788,26 @@ soup_uri_uses_default_port (SoupURI *uri)
 void
 soup_uri_set_scheme (SoupURI *uri, const char *scheme)
 {
-	uri->scheme = soup_uri_get_scheme (scheme, strlen (scheme));
+	uri->scheme = soup_uri_parse_scheme (scheme, strlen (scheme));
 	uri->port = soup_scheme_default_port (uri->scheme);
 }
 
+/**
+ * soup_uri_get_user:
+ * @uri: a #SoupURI
+ *
+ * Gets @uri's user.
+ *
+ * Return value: @uri's user.
+ *
+ * Since: 2.32
+ **/
+const char *
+soup_uri_get_user (SoupURI *uri)
+{
+	return uri->user;
+}
+
 /**
  * soup_uri_set_user:
  * @uri: a #SoupURI
@@ -805,6 +822,22 @@ soup_uri_set_user (SoupURI *uri, const char *user)
 	uri->user = g_strdup (user);
 }
 
+/**
+ * soup_uri_get_password:
+ * @uri: a #SoupURI
+ *
+ * Gets @uri's password.
+ *
+ * Return value: @uri's password.
+ *
+ * Since: 2.32
+ **/
+const char *
+soup_uri_get_password (SoupURI *uri)
+{
+	return uri->password;
+}
+
 /**
  * soup_uri_set_password:
  * @uri: a #SoupURI
@@ -819,6 +852,22 @@ soup_uri_set_password (SoupURI *uri, const char *password)
 	uri->password = g_strdup (password);
 }
 
+/**
+ * soup_uri_get_host:
+ * @uri: a #SoupURI
+ *
+ * Gets @uri's host.
+ *
+ * Return value: @uri's host.
+ *
+ * Since: 2.32
+ **/
+const char *
+soup_uri_get_host (SoupURI *uri)
+{
+	return uri->host;
+}
+
 /**
  * soup_uri_set_host:
  * @uri: a #SoupURI
@@ -837,6 +886,22 @@ soup_uri_set_host (SoupURI *uri, const char *host)
 	uri->host = g_strdup (host);
 }
 
+/**
+ * soup_uri_get_port:
+ * @uri: a #SoupURI
+ *
+ * Gets @uri's port.
+ *
+ * Return value: @uri's port.
+ *
+ * Since: 2.32
+ **/
+guint
+soup_uri_get_port (SoupURI *uri)
+{
+	return uri->port;
+}
+
 /**
  * soup_uri_set_port:
  * @uri: a #SoupURI
@@ -851,6 +916,22 @@ soup_uri_set_port (SoupURI *uri, guint port)
 	uri->port = port;
 }
 
+/**
+ * soup_uri_get_path:
+ * @uri: a #SoupURI
+ *
+ * Gets @uri's path.
+ *
+ * Return value: @uri's path.
+ *
+ * Since: 2.32
+ **/
+const char *
+soup_uri_get_path (SoupURI *uri)
+{
+	return uri->path;
+}
+
 /**
  * soup_uri_set_path:
  * @uri: a #SoupURI
@@ -865,6 +946,22 @@ soup_uri_set_path (SoupURI *uri, const char *path)
 	uri->path = g_strdup (path);
 }
 
+/**
+ * soup_uri_get_query:
+ * @uri: a #SoupURI
+ *
+ * Gets @uri's query.
+ *
+ * Return value: @uri's query.
+ *
+ * Since: 2.32
+ **/
+const char *
+soup_uri_get_query (SoupURI *uri)
+{
+	return uri->query;
+}
+
 /**
  * soup_uri_set_query:
  * @uri: a #SoupURI
@@ -882,7 +979,7 @@ soup_uri_set_query (SoupURI *uri, const char *query)
 /**
  * soup_uri_set_query_from_form:
  * @uri: a #SoupURI
- * @form: a #GHashTable containing HTML form information
+ * @form: (element-type utf8 utf8): a #GHashTable containing HTML form information
  *
  * Sets @uri's query to the result of encoding @form according to the
  * HTML form rules. See soup_form_encode_hash() for more information.
@@ -918,6 +1015,22 @@ soup_uri_set_query_from_fields (SoupURI    *uri,
 	va_end (args);
 }
 
+/**
+ * soup_uri_get_fragment:
+ * @uri: a #SoupURI
+ *
+ * Gets @uri's fragment.
+ *
+ * Return value: @uri's fragment.
+ *
+ * Since: 2.32
+ **/
+const char *
+soup_uri_get_fragment (SoupURI *uri)
+{
+	return uri->fragment;
+}
+
 /**
  * soup_uri_set_fragment:
  * @uri: a #SoupURI
